@@ -2,6 +2,7 @@
  * interface.c: virInterface methods
  *
  * Copyright (C) 2010 Red Hat Inc.
+ * Copyright (C) 2013 Chris Lalancette <clalancette@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,63 +29,71 @@
 #if HAVE_TYPE_VIRINTERFACEPTR
 static VALUE c_interface;
 
-static void interface_free(void *i) {
-    generic_free(Interface, i);
+static void interface_free(void *i)
+{
+    ruby_libvirt_free_struct(Interface, i);
 }
 
-static virInterfacePtr interface_get(VALUE s) {
-    generic_get(Interface, s);
+static virInterfacePtr interface_get(VALUE i)
+{
+    ruby_libvirt_get_struct(Interface, i);
 }
 
-VALUE interface_new(virInterfacePtr i, VALUE conn) {
-    return generic_new(c_interface, i, conn, interface_free);
+VALUE ruby_libvirt_interface_new(virInterfacePtr i, VALUE conn)
+{
+    return ruby_libvirt_new_class(c_interface, i, conn, interface_free);
 }
 
 /*
  * call-seq:
  *   interface.undefine -> nil
  *
- * Call +virInterfaceUndefine+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceUndefine]
+ * Call virInterfaceUndefine[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceUndefine]
  * to undefine this interface.
  */
-static VALUE libvirt_interface_undefine(VALUE s) {
-    gen_call_void(virInterfaceUndefine, conn(s), interface_get(s));
+static VALUE libvirt_interface_undefine(VALUE i)
+{
+    ruby_libvirt_generate_call_nil(virInterfaceUndefine,
+                                   ruby_libvirt_connect_get(i),
+                                   interface_get(i));
 }
 
 /*
  * call-seq:
  *   interface.create(flags=0) -> nil
  *
- * Call +virInterfaceCreate+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceCreate]
+ * Call virInterfaceCreate[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceCreate]
  * to start this interface.
  */
-static VALUE libvirt_interface_create(int argc, VALUE *argv, VALUE s) {
+static VALUE libvirt_interface_create(int argc, VALUE *argv, VALUE i)
+{
     VALUE flags;
 
     rb_scan_args(argc, argv, "01", &flags);
-    if (NIL_P(flags))
-        flags = INT2NUM(0);
 
-    gen_call_void(virInterfaceCreate, conn(s), interface_get(s),
-                  NUM2UINT(flags));
+    ruby_libvirt_generate_call_nil(virInterfaceCreate,
+                                   ruby_libvirt_connect_get(i),
+                                   interface_get(i),
+                                   ruby_libvirt_value_to_uint(flags));
 }
 
 /*
  * call-seq:
  *   interface.destroy(flags=0) -> nil
  *
- * Call +virInterfaceDestroy+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceDestroy]
+ * Call virInterfaceDestroy[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceDestroy]
  * to shutdown this interface.
  */
-static VALUE libvirt_interface_destroy(int argc, VALUE *argv, VALUE s) {
+static VALUE libvirt_interface_destroy(int argc, VALUE *argv, VALUE i)
+{
     VALUE flags;
 
     rb_scan_args(argc, argv, "01", &flags);
-    if (NIL_P(flags))
-        flags = INT2NUM(0);
 
-    gen_call_void(virInterfaceDestroy, conn(s), interface_get(s),
-                  NUM2UINT(flags));
+    ruby_libvirt_generate_call_nil(virInterfaceDestroy,
+                                   ruby_libvirt_connect_get(i),
+                                   interface_get(i),
+                                   ruby_libvirt_value_to_uint(flags));
 }
 
 #if HAVE_VIRINTERFACEISACTIVE
@@ -92,11 +101,14 @@ static VALUE libvirt_interface_destroy(int argc, VALUE *argv, VALUE s) {
  * call-seq:
  *   interface.active? -> [true|false]
  *
- * Call +virInterfaceIsActive+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceIsActive]
+ * Call virInterfaceIsActive[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceIsActive]
  * to determine if this interface is currently active.
  */
-static VALUE libvirt_interface_active_p(VALUE p) {
-    gen_call_truefalse(virInterfaceIsActive, conn(p), interface_get(p));
+static VALUE libvirt_interface_active_p(VALUE p)
+{
+    ruby_libvirt_generate_call_truefalse(virInterfaceIsActive,
+                                         ruby_libvirt_connect_get(p),
+                                         interface_get(p));
 }
 #endif
 
@@ -104,59 +116,66 @@ static VALUE libvirt_interface_active_p(VALUE p) {
  * call-seq:
  *   interface.name -> string
  *
- * Call +virInterfaceGetName+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetName]
+ * Call virInterfaceGetName[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetName]
  * to retrieve the name of this interface.
  */
-static VALUE libvirt_interface_name(VALUE s) {
-    gen_call_string(virInterfaceGetName, conn(s), 0, interface_get(s));
+static VALUE libvirt_interface_name(VALUE i)
+{
+    ruby_libvirt_generate_call_string(virInterfaceGetName,
+                                      ruby_libvirt_connect_get(i), 0,
+                                      interface_get(i));
 }
 
 /*
  * call-seq:
  *   interface.mac -> string
  *
- * Call +virInterfaceGetMACString+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetMACString]
+ * Call virInterfaceGetMACString[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetMACString]
  * to retrieve the MAC address of this interface.
  */
-static VALUE libvirt_interface_mac(VALUE s) {
-    gen_call_string(virInterfaceGetMACString, conn(s), 0, interface_get(s));
+static VALUE libvirt_interface_mac(VALUE i)
+{
+    ruby_libvirt_generate_call_string(virInterfaceGetMACString,
+                                      ruby_libvirt_connect_get(i),
+                                      0, interface_get(i));
 }
 
 /*
  * call-seq:
  *   interface.xml_desc -> string
  *
- * Call +virInterfaceGetXMLDesc+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetXMLDesc]
+ * Call virInterfaceGetXMLDesc[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceGetXMLDesc]
  * to retrieve the XML of this interface.
  */
-static VALUE libvirt_interface_xml_desc(int argc, VALUE *argv, VALUE s) {
+static VALUE libvirt_interface_xml_desc(int argc, VALUE *argv, VALUE i)
+{
     VALUE flags;
 
     rb_scan_args(argc, argv, "01", &flags);
 
-    if (NIL_P(flags))
-        flags = INT2NUM(0);
-
-    gen_call_string(virInterfaceGetXMLDesc, conn(s), 1, interface_get(s),
-                    NUM2UINT(flags));
+    ruby_libvirt_generate_call_string(virInterfaceGetXMLDesc,
+                                      ruby_libvirt_connect_get(i),
+                                      1, interface_get(i),
+                                      ruby_libvirt_value_to_uint(flags));
 }
 
 /*
  * call-seq:
  *   interface.free -> nil
  *
- * Call +virInterfaceFree+[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceFree]
+ * Call virInterfaceFree[http://www.libvirt.org/html/libvirt-libvirt.html#virInterfaceFree]
  * to free this interface.  The object will no longer be valid after this call.
  */
-static VALUE libvirt_interface_free(VALUE s) {
-    gen_call_free(Interface, s);
+static VALUE libvirt_interface_free(VALUE i)
+{
+    ruby_libvirt_generate_call_free(Interface, i);
 }
 #endif
 
 /*
  * Class Libvirt::Interface
  */
-void init_interface()
+void ruby_libvirt_interface_init(void)
 {
 #if HAVE_TYPE_VIRINTERFACEPTR
     c_interface = rb_define_class_under(m_libvirt, "Interface", rb_cObject);
