@@ -7,6 +7,8 @@ $: << File.dirname(__FILE__)
 require 'libvirt'
 require 'test_utils.rb'
 
+set_test_object("nwfilter")
+
 conn = Libvirt::open("qemu:///system")
 
 # TESTGROUP: nwfilter.undefine
@@ -51,6 +53,8 @@ expect_too_many_args(newnw, "free", 1)
 
 expect_success(newnw, "no args", "free")
 
+# END TESTS
+
 conn.close
 
 finish_tests
